% demo.m
% 
% Please be patient. It may take a few seconds to process a median filtered
% image of size 512*512.
% 

clear;
addpath('kerest'); % for kernel estimation
addpath('measures'); % scalar-based median filtering forensics

%%%%%%%%%%%%%%%%%%%%%%%%%% image quality enhancement %%%%%%%%%%%%%%%%%%%%%%
% parameter setting
opts.winS = 3; % window size
opts.kernel = fspecial('average',opts.winS); % AVE kernel 
opts.init_kernel = [];
opts.iDither = false;
opts.nb_omega = 0.4;
opts.nb_lambda = 3000;
opts.nb_gamma = 200;

% processing
imgname = 'mfte00880.pgm';
mfimgname = 'mfte00880_mf3.pgm';
oriI = imread(imgname); % original image
mfI = imread(mfimgname); % MF image
pI = uint8(MFDCONV(mfI, opts)); % processed image, with quality enhancement

% results
fprintf('mfI\t%10.4f\t%10.4f\n',psnr(oriI,mfI),ssim(oriI,mfI));
fprintf('pI\t%10.4f\t%10.4f\n\n',psnr(oriI,pI),ssim(oriI,pI));

%%%%%%%%%%%%%%%%%%%%%%%%%% anti-forensics %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% parameter setting
opts.winS = 3;
opts.kernel = [];
opts.init_kernel = fspecial('average',opts.winS); % AVEE kernel
opts.iDither = true; % pixel value perturbation
opts.nb_omega = 0.1;
opts.nb_lambda = 1500;
opts.nb_gamma = 500;

% processing
imgname = 'mfte00880.pgm';
mfimgname = 'mfte00880_mf3.pgm';
oriI = imread(imgname); % original image
mfI = imread(mfimgname); % MF image
afI = uint8(MFDCONV(mfI, opts)); % MF anti-forensic image

% results
fprintf('oriI\t%10.4f\t%10.4f\t%10.4f\t%10.4f\t%10.4f\t%10.4f\n',psnr(oriI,oriI),ssim(oriI,oriI),kirchnerSPIE10(oriI,1,0),kirchnerSPIE10(oriI,1,0,64),caoICME10(oriI,7,100),mff44(oriI,true));
fprintf('mfI\t%10.4f\t%10.4f\t%10.4f\t%10.4f\t%10.4f\t%10.4f\n',psnr(oriI,mfI),ssim(oriI,mfI),kirchnerSPIE10(mfI,1,0),kirchnerSPIE10(mfI,1,0,64),caoICME10(mfI,7,100),mff44(mfI,true));
fprintf('afI\t%10.4f\t%10.4f\t%10.4f\t%10.4f\t%10.4f\t%10.4f\n',psnr(oriI,afI),ssim(oriI,afI),kirchnerSPIE10(afI,1,0),kirchnerSPIE10(afI,1,0,64),caoICME10(afI,7,100),mff44(afI,true));
